==> taler-wallet-v0.9.4a.tar.gz: taler-wallet-v0.9.4a/packages/taler-harness/src/integrationtests/test-wallet-insufficient-balance.ts <==
/*
 This file is part of GNU Taler
 (C) 2020 Taler Systems S.A.

 GNU Taler is free software; you can redistribute it and/or modify it under the
 terms of the GNU General Public License as published by the Free Software
 Foundation; either version 3, or (at your option) any later version.

 GNU Taler is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with
 GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
 */

/**
 * Imports.
 */
import {
  AmountString,
  Duration,
  PaymentInsufficientBalanceDetails,
  TalerErrorCode,
  WalletNotification,
} from "@gnu-taler/taler-util";
import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
import { CoinConfig, defaultCoinConfig } from "../harness/denomStructures.js";
import {
  ExchangeService,
  FakebankService,
  GlobalTestState,
  MerchantService,
  WalletClient,
  WalletService,
  generateRandomPayto,
  setupDb,
} from "../harness/harness.js";
import { withdrawViaBankV2 } from "../harness/helpers.js";

export async function runWalletInsufficientBalanceTest(t: GlobalTestState) {
  // Set up test environment

  const db = await setupDb(t);

  const bank = await FakebankService.create(t, {
    allowRegistrations: true,
    currency: "TESTKUDOS",
    database: db.connStr,
    httpPort: 8082,
  });

  const exchange = ExchangeService.create(t, {
    name: "testexchange-1",
    currency: "TESTKUDOS",
    httpPort: 8081,
    database: db.connStr,
  });

  const merchant = await MerchantService.create(t, {
    name: "testmerchant-1",
    currency: "TESTKUDOS",
    httpPort: 8083,
    database: db.connStr,
  });

  const exchangeBankAccount = await bank.createExchangeAccount(
    "myexchange",
    "x",
  );
  exchangeBankAccount.skipWireFeeCreation = true;
  exchange.addBankAccount("1", exchangeBankAccount);

  bank.setSuggestedExchange(exchange, exchangeBankAccount.accountPaytoUri);

  await bank.start();

  await bank.pingUntilAvailable();

  const coinConfig: CoinConfig[] = defaultCoinConfig.map((x) => x("TESTKUDOS"));
  exchange.addCoinConfigList(coinConfig);

  await exchange.start();
  await exchange.pingUntilAvailable();

  merchant.addExchange(exchange);

  await merchant.start();
  await merchant.pingUntilAvailable();

  await merchant.addInstanceWithWireAccount({
    id: "default",
    name: "Default Instance",
    paytoUris: [generateRandomPayto("merchant-default")],
    defaultWireTransferDelay: Duration.toTalerProtocolDuration(
      Duration.fromSpec({ minutes: 1 }),
    ),
  });

  await merchant.addInstanceWithWireAccount({
    id: "minst1",
    name: "minst1",
    paytoUris: [generateRandomPayto("minst1")],
    defaultWireTransferDelay: Duration.toTalerProtocolDuration(
      Duration.fromSpec({ minutes: 1 }),
    ),
  });

  const walletService = new WalletService(t, {
    name: "wallet",
    useInMemoryDb: true,
  });
  await walletService.start();
  await walletService.pingUntilAvailable();

  const allNotifications: WalletNotification[] = [];

  const walletClient = new WalletClient({
    name: "wallet",
    unixPath: walletService.socketPath,
    onNotification(n) {
      console.log("got notification", n);
      allNotifications.push(n);
    },
  });
  await walletClient.connect();
  await walletClient.client.call(WalletApiOperation.InitWallet, {
    config: {
      testing: {
        skipDefaults: true,
      },
    },
  });

  const wres = await withdrawViaBankV2(t, {
    amount: "TESTKUDOS:10",
    bank,
    exchange,
    walletClient,
  });
  await wres.withdrawalFinishedCond;

  const exc = await t.assertThrowsTalerErrorAsync(async () => {
    await walletClient.call(WalletApiOperation.PrepareDeposit, {
      amount: "TESTKUDOS:5" as AmountString,
      depositPaytoUri: "payto://x-taler-bank/localhost/foobar",
    });
  });
  t.assertDeepEqual(
    exc.errorDetail.code,
    TalerErrorCode.WALLET_DEPOSIT_GROUP_INSUFFICIENT_BALANCE,
  );
  const insufficientBalanceDetails: PaymentInsufficientBalanceDetails =
    exc.errorDetail.insufficientBalanceDetails;

  t.assertAmountEquals(
    insufficientBalanceDetails.balanceAvailable,
    "TESTKUDOS:9.72",
  );
  t.assertAmountEquals(
    insufficientBalanceDetails.balanceExchangeDepositable,
    "TESTKUDOS:0",
  );
}

runWalletInsufficientBalanceTest.suites = ["wallet"];

==> taler-wallet-v0.12.7.tar.gz: taler-wallet-v0.12.7/packages/taler-harness/src/integrationtests/test-wallet-insufficient-balance.ts <==
/*
 This file is part of GNU Taler
 (C) 2020 Taler Systems S.A.

 GNU Taler is free software; you can redistribute it and/or modify it under the
 terms of the GNU General Public License as published by the Free Software
 Foundation; either version 3, or (at your option) any later version.

 GNU Taler is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with
 GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
 */

/**
 * Imports.
 */
import {
  AmountString,
  Duration,
  PaymentInsufficientBalanceDetails,
  TalerErrorCode,
} from "@gnu-taler/taler-util";
import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
import { CoinConfig, defaultCoinConfig } from "../harness/denomStructures.js";
import {
  GlobalTestState,
  generateRandomPayto,
  setupDb,
} from "../harness/harness.js";
import { createSimpleTestkudosEnvironmentV3, withdrawViaBankV3 } from "../harness/helpers.js";

export async function runWalletInsufficientBalanceTest(t: GlobalTestState) {
  // Set up test environment

  const db = await setupDb(t);

  const coinConfig: CoinConfig[] = defaultCoinConfig.map((x) => x("TESTKUDOS"));

  let {
    bankClient,
    exchange,
    merchant,
    walletService,
    walletClient,
  } = await createSimpleTestkudosEnvironmentV3(t, coinConfig, {
    skipWireFeeCreation: true,
  });

  await merchant.addInstanceWithWireAccount({
    id: "default",
    name: "Default Instance",
    paytoUris: [generateRandomPayto("merchant-default")],
    defaultWireTransferDelay: Duration.toTalerProtocolDuration(
      Duration.fromSpec({ minutes: 1 }),
    ),
  });

  await merchant.addInstanceWithWireAccount({
    id: "minst1",
    name: "minst1",
    paytoUris: [generateRandomPayto("minst1")],
    defaultWireTransferDelay: Duration.toTalerProtocolDuration(
      Duration.fromSpec({ minutes: 1 }),
    ),
  });

  await walletClient.client.call(WalletApiOperation.InitWallet, {
    config: {
      testing: {
        skipDefaults: true,
      },
    },
  });

  const wres = await withdrawViaBankV3(t, {
    amount: "TESTKUDOS:10",
    bankClient,
    exchange,
    walletClient,
  });
  await wres.withdrawalFinishedCond;

  const exc = await t.assertThrowsTalerErrorAsync(async () => {
    await walletClient.call(WalletApiOperation.PrepareDeposit, {
      amount: "TESTKUDOS:5" as AmountString,
      depositPaytoUri: "payto://x-taler-bank/localhost/foobar",
    });
  });

  t.assertDeepEqual(
    exc.errorDetail.code,
    TalerErrorCode.WALLET_DEPOSIT_GROUP_INSUFFICIENT_BALANCE,
  );

  const insufficientBalanceDetails: PaymentInsufficientBalanceDetails =
    exc.errorDetail.insufficientBalanceDetails;

  t.assertAmountEquals(
    insufficientBalanceDetails.balanceAvailable,
    "TESTKUDOS:9.72",
  );
  t.assertAmountEquals(
    insufficientBalanceDetails.balanceExchangeDepositable,
    "TESTKUDOS:0",
  );
}

runWalletInsufficientBalanceTest.suites = ["wallet"];
